# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

import json
import os
from pathlib import Path
import shutil
from typing import Union

from ansys.optislang.core import Optislang


def dump_project_state(project_file: Path, project_state_file: Path) -> None:
    """Start PyOptiSLang and dump project state file."""

    working_directory = project_file.parent
    temporary_directory = working_directory / "extract_project_tree.tmp"

    if not os.path.exists(temporary_directory):
        os.makedirs(temporary_directory)
    else:
        os.rmdir(temporary_directory)
        os.makedirs(temporary_directory)

    shutil.copy(project_file, temporary_directory / project_file.name)

    osl = Optislang(
        project_path=temporary_directory / project_file.name,
        loglevel="INFO",
        reset=True,
        shutdown_on_finished=True,
        dump_project_state=project_state_file,
        ini_timeout=300,
    )

    osl.dispose()

    if not project_state_file.exists():
        raise Exception("The project state file has not been generated.")

    shutil.rmtree(temporary_directory)


def get_project_tree(project_state_file: Path) -> dict:
    """Read the project tree of an optiSLang project."""

    with open(Path(project_state_file).absolute()) as f:
        response = json.load(f)

    # Initialize project tree with default steps.
    project_tree = [
        {
            "key": "problem_setup_step",
            "text": "Problem Setup",
            "depth": 0,
            "uid": None,
            "type": None,
            "kind": None,
            "is_root": False,
        },
    ]

    root_system = response["projects"][0]["system"]

    # Declare root.
    project_tree.extend(
        [
            {
                "key": f'{root_system["name"].lower()}_{root_system["uid"]}',
                "text": f'{root_system["name"]} (root)',
                "depth": 0,
                "uid": root_system["uid"],
                "type": root_system["type"],
                "kind": root_system["kind"],
                "is_root": True,
            },
        ]
    )

    # Declare nodes.
    for node in root_system["nodes"]:
        depth = 1
        project_tree.append(
            {
                "key": f'{node["name"].lower()}_{node["uid"]}',
                "text": node["name"],
                "depth": depth,
                "uid": node["uid"],
                "type": node["type"],
                "kind": node["kind"],
                "is_root": False,
            }
        )
        if "nodes" in node.keys():
            project_tree.extend(_get_node_tree(node["nodes"], depth=depth))

    return {"project_tree": project_tree}


def _get_node_tree(data: Union[list, dict], depth: int = 1) -> list:

    if isinstance(data, dict):
        node_tree = []
        depth += 1
        node_tree.append(
            {
                "key": f'{data["name"].lower()}_{data["uid"]}',
                "text": data["name"],
                "depth": depth,
                "uid": data["uid"],
                "type": data["type"],
                "kind": data["kind"],
                "is_root": False,
            }
        )
        if "nodes" in data.keys():
            node_tree.extend(_get_node_tree(item, depth=1))
        return node_tree
    elif isinstance(data, list):
        node_tree = []
        for item in data:
            node_tree.extend(_get_node_tree(item, depth=depth))
        return node_tree


def get_step_list(project_tree: dict) -> list:
    """Return a list of steps to feed the AnsysDashTreeview component."""

    return project_tree["project_tree"]


def get_node_list(project_tree: dict) -> list:
    """Return the list of nodes in the optiSLang project file."""

    return [
        node_info
        for node_info in project_tree["project_tree"]
        if node_info["key"] not in ["problem_setup_step"]
    ]



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the project summary step."""

import dash_bootstrap_components as dbc
from dash_extensions.enrich import html

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.components.project_summary_table import ProjectSummaryTable


def layout(project_status_info: dict, font_size: str = "15px") -> html.Div:
    """Layout of the project summary view."""

    project_summary_table = ProjectSummaryTable()

    if project_status_info:
        project_summary_table.project_status_info = project_status_info

    return html.Div(
        [
            html.Br(),
            dbc.Row(
                [
                    dbc.Col(
                        dbc.Card(
                            [
                                dbc.CardBody(
                                    [
                                        html.H4("Project Information", className="card-title"),
                                        project_summary_table.render(),
                                    ]
                                ),
                            ],
                            color="warning",
                            outline=True,
                        ),
                        width=12,
                    ),
                ]
            ),
        ]
    )



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Provide a component to handle the actor logs for system and actor node types."""

from datetime import datetime

from dash import dash_table
import pandas as pd

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.utils.common_functions import (
    remove_key_from_dictionaries,
    sort_dict_by_ordered_keys,
)


class ActorLogsTable:
    """Actor logs component."""

    def __init__(self) -> None:
        """Constructor."""

        self.actor_info: dict = None
        self.font_size: str = "15px"

    def _get_data(self) -> pd.DataFrame:

        has_data = False

        if self.actor_info:
            if "log_messages" in self.actor_info.keys():
                if len(self.actor_info["log_messages"]):
                    has_data = True
                    # Remove hid key from list dictionaries because it is useless for UI
                    # and prevent transformation to DataFrame.
                    actor_logs_data = remove_key_from_dictionaries(self.actor_info["log_messages"], "hid")
                    # Transform list of dictionaries into dictionary
                    actor_logs_data = {
                        key: [d[key] for d in self.actor_info["log_messages"]]
                        for key in self.actor_info["log_messages"][0]
                    }
                    # Sort keys in order
                    actor_logs_data = sort_dict_by_ordered_keys(actor_logs_data, ["time_stamp", "level", "message"])
                    # Rename keys
                    actor_logs_data["Time"] = actor_logs_data.pop("time_stamp")
                    actor_logs_data["Level"] = actor_logs_data.pop("level")
                    actor_logs_data["Message"] = actor_logs_data.pop("message")
                    # Convert timestamp
                    for index, value in enumerate(actor_logs_data["Time"]):
                        dt = datetime.strptime(value, "%Y%m%dT%H%M%S.%f")
                        actor_logs_data["Time"][index] = dt.strftime("%Y-%m-%d %H-%M-%S-%f")[:-3]

        if not has_data:
            actor_logs_data = {"Time": [], "Level": [], "Message": []}

        return pd.DataFrame(actor_logs_data)

    def render(self):
        """Generate table."""

        data = self._get_data()

        return dash_table.DataTable(
            data=data.to_dict("records"),
            columns=[{"name": i, "id": i, "type": "text"} for i in data.columns],
            fixed_rows={"headers": True},
            style_header={"font_family": "Roboto", "font_size": self.font_size, "fontWeight": "bold"},
            style_cell={
                "textAlign": "left",
                "font_family": "Roboto",
                "font_size": self.font_size,
            },
            style_cell_conditional=[
                {"if": {"column_id": "Time"}, "minWidth": "60px", "maxWidth": "60px", "width": "60px"},
                {
                    "if": {"column_id": "Level"},
                    "minWidth": "30px",
                    "maxWidth": "30px",
                    "width": "30px",
                    "textAlign": "center",
                },
                {"if": {"column_id": "Message"}, "minWidth": "200px", "maxWidth": "200px", "width": "200px"},
            ],
            style_data_conditional=[
                {
                    "if": {"column_id": "Level", "filter_query": '{Level} eq "INFO"'},
                    "backgroundColor": "rgb(227, 245, 252)",
                    "color": "rgb(0, 0, 0)",
                    "textAlign": "center",
                }
            ],
            style_as_list_view=True,
        )



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Initialization of the frontend layout across all the steps."""

from ansys.saf.glow.client.dashclient import DashClient
from ansys_dash_treeview import AnsysDashTreeview
import dash_bootstrap_components as dbc
from dash_extensions.enrich import Input, Output, State, callback, callback_context, dcc, html
from dash.exceptions import PreventUpdate
from dash_iconify import DashIconify
import dash_loading_spinners as dls
from pathlib import Path

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.definition import {{ cookiecutter.__solution_definition_name }}
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.pages import monitoring_page, problem_setup_page
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.utils.common_functions import extract_dict_by_key, read_system_hierarchy


layout = html.Div(
    [
        dcc.Location(id="url", refresh=False),  # represents the browser address bar and doesn't render anything
        dls.Pulse(
            color="#FFB71B",
            speed_multiplier=0.4,
            margin=4,
            width=57,
            fullscreen=True,
            fullscreen_style={
                "background-color": "rgba(255, 255, 255, 0)",
                "position": "relative"
            },
            children=html.Div(id="wait_app_initialization"),
        ),
        html.Div(
            id="page_content",
        ),
        dbc.Button(
            id="dummy_button",
            style={"display": "none"},
            n_clicks=0,
            color="dark"
        ),
    ]
)


@callback(
    Output("return_to_portal", "children"),
    Input("url", "pathname"),
)
def return_to_portal(pathname):
    """Display Solution Portal when back-to-portal button gets selected."""
    portal_ui_url = DashClient.get_portal_ui_url()
    children = (
        []
        if portal_ui_url is None
        else [
            dbc.Button(
                "Back to Projects",
                id="return_to_portal",
                className="me-2",
                n_clicks=0,
                href=portal_ui_url,
                style={"background-color": "rgba(0, 0, 0, 1)", "border-color": "rgba(0, 0, 0, 1)"},
            )
        ]
    )
    return children


@callback(
    Output("page_content", "children"),
    Output("dummy_button", "n_clicks"),
    Output("wait_app_initialization", "children"),
    Input("url", "pathname"),
    State("dummy_button", "n_clicks")
)
def project_initialization(pathname, n_clicks):

    project = DashClient[{{ cookiecutter.__solution_definition_name }}].get_project(pathname)

    if not project.steps.problem_setup_step.project_initialized:
        # Solution-specific code
        project_tree_path = Path(__file__).absolute().parent.parent.parent / "model" / "assets" / "project_state.json"
        if not project_tree_path.exists():
            project.steps.problem_setup_step.generate_project_state()
        # Project-specific code
        project.steps.problem_setup_step.upload_bulk_files_to_project_directory()
        project.steps.problem_setup_step.read_project_tree()
        project.steps.problem_setup_step.get_app_metadata()
        project.steps.problem_setup_step.check_ansys_ecosystem()
        return [
            dbc.Stack(
                [
                    html.Div(
                        [html.Img(src=r"/assets/logos/ansys-solutions-horizontal-logo.png", style={"width": "80%"})],
                    ),
                    html.Div(
                        [
                            dbc.Button(
                                f"Project Name: {project.project_display_name}",
                                id="project-name",
                                disabled=True,
                                style={
                                    "color": "rgba(0, 0, 0, 1)",
                                    "background-color": "rgba(255, 255, 255, 1)",
                                    "border-color": "rgba(0, 0, 0, 1)",
                                },
                            )
                        ],
                        className="ms-auto",
                    ),
                    html.Div(id="return_to_portal"),
                ],
                direction="horizontal",
                gap=3,
            ),
            html.Br(),
            dbc.Row(
                [
                    dbc.Col(
                        AnsysDashTreeview(
                            id="navigation_tree",
                            items=[
                                {
                                    "key": "problem_setup_step",
                                    "text": "Problem Setup",
                                    "depth": 0,
                                    "uid": None,
                                    "type": None,
                                    "kind": None,
                                    "is_root": False,
                                },
                            ],
                            children=[
                                DashIconify(icon="bi:caret-right-square-fill"),
                                DashIconify(icon="bi:caret-down-square-fill"),
                            ],
                            style={"showButtons": True, "focusColor": "#ffb71b", "itemHeight": "32"},  # Ansys gold
                        ),
                        width=2,
                        style={"background-color": "rgba(242, 242, 242, 0.6)"},  # Ansys grey
                    ),
                    dbc.Col(html.Div(id="page_body", style={"padding-right": "1%"}), width=10),
                ],
            ),
        ], n_clicks + 1, True
    else:
        raise PreventUpdate


@callback(
    Output("navigation_tree", "items"),
    Input("dummy_button", "n_clicks"),
    Input("url", "pathname"),
)
def display_step_list(n_clicks, pathname):
    """Display current project name."""
    project = DashClient[{{ cookiecutter.__solution_definition_name }}].get_project(pathname)

    return project.steps.problem_setup_step.step_list


@callback(
    Output("page_body", "children"),
    [
        Input("url", "pathname"),
        Input("navigation_tree", "focus"),
        Input("dummy_button", "n_clicks"),
    ],
    prevent_initial_call=True,
)
def display_page_body(pathname, value, n_clicks):
    """
    Display page content.

    this callback is essential for initializing the step based on the persisted
    state of the project when the browser first displays the project to the user
    given the project's URL
    """

    project = DashClient[{{ cookiecutter.__solution_definition_name }}].get_project(pathname)
    triggered_id = callback_context.triggered[0]["prop_id"].split(".")[0]

    if triggered_id == "url" or triggered_id == "dummy_button":
        return problem_setup_page.layout(project.steps.problem_setup_step)
    if triggered_id == "navigation_tree":
        if value is None:
            page_layout = html.H1("Welcome!")
        elif value == "problem_setup_step":
            page_layout = problem_setup_page.layout(project.steps.problem_setup_step)
        else:
            node_info = extract_dict_by_key(project.steps.problem_setup_step.step_list, "key", value, expect_unique=True)
            page_layout = monitoring_page.layout(
                project.steps.problem_setup_step, project.steps.monitoring_step, node_info
            )
        return page_layout



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the results file view."""

from dash_extensions.enrich import html


def layout(project_status_info: dict, uid: str) -> html.Div:
    """Layout of the result files view."""

    return html.Div([])



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the monitoring step."""

from ansys.saf.glow.client.dashclient import DashClient
import dash_bootstrap_components as dbc
from dash_extensions.enrich import Input, Output, State, callback, dcc, html

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.definition import {{ cookiecutter.__solution_definition_name }}
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.monitoring_step import MonitoringStep
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.problem_setup_step import ProblemSetupStep
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.components.logs_table import LogsTable
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.utils.alerts import update_monitoring_alert
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.utils.common_functions import update_list_of_tabs
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.views import (
    design_table_page,
    project_summary_page,
    result_files_page,
    scenery_page,
    status_overview_page,
    summary_page,
    visualization_page,
)


def layout(problem_setup_step: ProblemSetupStep, monitoring_step: MonitoringStep, node_info: dict) -> html.Div:

    list_of_tabs = update_list_of_tabs(node_info)

    return html.Div(
        [
            html.Div(
                id="monitoring_alert",
                children=update_monitoring_alert(problem_setup_step),
            ),
            dbc.Tabs(
                list_of_tabs,
                id="monitoring_tabs",
                active_tab=list_of_tabs[0].tab_id,
                style={
                    "color": "#000000",
                    "background-color": "#FFFFFF",
                    "text-color": "#000000",
                },
            ),
            html.Div(
                id="monitoring_page_content",
            ),
            dcc.Store(id="node_info", data=node_info),
            html.Br(),
            html.Div(
                [
                    dbc.Button(
                        "optiSLang logs",
                        id="fade_button",
                        className="mb-3",
                        n_clicks=0,
                        style={"background-color": "#000000", "borderColor": "#000000"},
                    ),
                    dbc.Fade(
                        id="fade",
                        is_in=False,
                        appear=False,
                    ),
                ]
            ),
            dcc.Interval(
                id="update_data_interval",
                interval=1 * 3000,  # in milliseconds
                n_intervals=0,
            ),
        ]
    )


@callback(
    Output("monitoring_page_content", "children"),
    Input("monitoring_tabs", "active_tab"),
    Input("update_data_interval", "n_intervals"),
    Input("url", "pathname"),
    State("node_info", "data"),
)
def update_page_content(active_tab, n_intervals, pathname, node_info):
    """Update the page content according to the selected tab."""

    project = DashClient[{{ cookiecutter.__solution_definition_name }}].get_project(pathname)
    problem_setup_step = project.steps.problem_setup_step

    if active_tab == "project_summary_tab":
        return project_summary_page.layout(problem_setup_step.project_status_info)
    elif active_tab == "summary_tab":
        return summary_page.layout(
            problem_setup_step.actors_info,
            problem_setup_step.actors_status_info,
            problem_setup_step.results_files,
            node_info["uid"],
        )
    elif active_tab == "result_files_tab":
        return result_files_page.layout(problem_setup_step.project_status_info, node_info["uid"])
    elif active_tab == "scenery_tab":
        return scenery_page.layout(problem_setup_step.project_status_info)
    elif active_tab == "design_table_tab":
        return design_table_page.layout(
            problem_setup_step.actors_info, problem_setup_step.actors_status_info, node_info["uid"]
        )
    elif active_tab == "visualization_tab":
        return visualization_page.layout(problem_setup_step.project_status_info, node_info["uid"])
    elif active_tab == "status_overview_tab":
        return status_overview_page.layout(problem_setup_step.project_status_info)


@callback(
    Output("fade", "children"),
    Output("fade", "is_in"),
    Input("fade_button", "n_clicks"),
    Input("url", "pathname"),
    State("fade", "is_in"),
)
def display_optislang_logs(n_clicks, pathname, is_in):

    project = DashClient[{{ cookiecutter.__solution_definition_name }}].get_project(pathname)
    problem_setup_step = project.steps.problem_setup_step

    if not n_clicks:
        # Button has never been clicked
        return None, False

    table = LogsTable(problem_setup_step.optislang_logs)

    return table.render(), not is_in




# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.


def update_placeholders(ui_values: list, placeholders: dict) -> dict:
    """Information needed."""

    updated_dict = {}
    for row in ui_values[1:]:
        parameter_name = row["props"]["children"][0]["props"]["children"]["props"]["children"]
        input_value = row["props"]["children"][1]["props"]["children"]["props"]["value"]

        placeholder_values = placeholders.get("placeholder_values")
        if parameter_name in placeholder_values:
            updated_dict[parameter_name] = input_value
    return updated_dict



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the problem setup step."""

import time

from ansys.saf.glow.client.dashclient import DashClient
from ansys.saf.glow.solution import MethodStatus
from ansys.solutions.dash_components.table import InputRow
from dash.exceptions import PreventUpdate
import dash_bootstrap_components as dbc
from dash_extensions.enrich import Input, Output, State, callback, dcc, html
from ansys.solutions.optislang.frontend_components.load_sections import to_dash_sections

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.definition import {{ cookiecutter.__solution_definition_name }}
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.problem_setup_step import ProblemSetupStep
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.utils.alerts import update_alerts
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.utils.placeholders import update_placeholders


def layout(problem_setup_step: ProblemSetupStep) -> html.Div:
    """Layout of the problem setup step."""

    # Upload placeholders and assets
    if problem_setup_step.placeholders == {}:
        problem_setup_step.get_default_placeholder_values()
        problem_setup_step.ui_placeholders = problem_setup_step.placeholders

    project_properties_sections = to_dash_sections(problem_setup_step.placeholders, problem_setup_step.registered_files)

    return html.Div(
        [
            # Header
            html.H1(
                problem_setup_step.app_metadata["title"].strip().title(),
                className="display-3",
                style={"font-size": "45px"},
            ),
            html.P(
                problem_setup_step.app_metadata["description"].strip(),
                className="lead",
                style={"font-size": "25px"},
            ),
            html.Hr(className="my-2"),
            html.Br(),
            # Alerts
            dbc.Row(
                [
                    dbc.Col(
                        [
                            dbc.Stack(
                                update_alerts(problem_setup_step),
                                id="alert_messages",
                                direction="horizontal",
                                gap=3,
                            ),
                        ],
                        width=12,
                    )
                ]
            ),
            html.Br(),
            # Input form
            dbc.Row(project_properties_sections),
            dbc.Row(
                [
                    dbc.Accordion(
                        [
                            dbc.AccordionItem(
                                [
                                    # InputRow(
                                    #     "button",
                                    #     "check_ansys_ecosystem",
                                    #     "Check Ansys ecosystem",
                                    #     disabled=False,
                                    #     label_width=2,
                                    #     value_width=4,
                                    #     unit_width=1,
                                    #     description_width=4,
                                    #     class_name="button",
                                    # ).get(),
                                    # dcc.Loading(
                                    #     type="circle",
                                    #     fullscreen=True,
                                    #     color="#ffb71b",
                                    #     style={
                                    #         "background-color": "rgba(55, 58, 54, 0.1)",
                                    #     },
                                    #     children=html.Div(id="wait_ecosystem_ckeck"),
                                    # ),
                                    # html.Br(),
                                    InputRow(
                                        "button",
                                        "start_analysis",
                                        "Start analysis",
                                        disabled=True
                                        if problem_setup_step.analysis_running
                                        or not problem_setup_step.ansys_ecosystem_ready
                                        else False,
                                        label_width=2,
                                        value_width=4,
                                        unit_width=1,
                                        description_width=4,
                                        class_name="button",
                                    ).get(),
                                    dcc.Loading(
                                        type="circle",
                                        fullscreen=True,
                                        color="#ffb71b",
                                        style={
                                            "background-color": "rgba(55, 58, 54, 0.1)",
                                        },
                                        children=html.Div(id="wait_start_analysis"),
                                    ),
                                    dcc.Interval(
                                        id="solve_interval_component",
                                        interval=1 * 3000,  # in milliseconds
                                        n_intervals=0,
                                    ),
                                ],
                                title="Start Analysis",
                                item_id="start_analysis_accordion",
                            ),
                        ]
                    )
                ]
            ),
        ]
    )


for alert in ["optislang_version", "optislang_solve"]:

    @callback(
        Output(f"popover_{alert}", "is_open"),
        [Input(f"popover_{alert}_target", "n_clicks")],
        [State(f"popover_{alert}", "is_open")],
    )
    def toggle_popover(n_clicks, is_open):
        if n_clicks:
            return not is_open
        return is_open


# @callback(
#     Output("alert_messages", "children"),
#     Output("wait_ecosystem_ckeck", "children"),
#     Output("start_analysis", "disabled"),
#     Input("check_ansys_ecosystem", "n_clicks"),
#     State("url", "pathname"),
#     prevent_initial_call=True,
# )
# def check_ansys_ecosystem(n_clicks, pathname):
#     """Start optiSLang and run the simulation. Wait for the process to complete."""

#     project = DashClient[{{ cookiecutter.__solution_definition_name }}].get_project(pathname)
#     problem_setup_step = project.steps.problem_setup_step

#     if n_clicks:
#         problem_setup_step.check_ansys_ecosystem()
#         return update_alerts(problem_setup_step), True, False if problem_setup_step.ansys_ecosystem_ready else True
#     else:
#         raise PreventUpdate


@callback(
    Output("alert_messages", "children"),
    Output("wait_start_analysis", "children"),
    Output("start_analysis", "disabled"),
    Input("start_analysis", "n_clicks"),
    [State("table-placeholders", "children")],
    State("url", "pathname"),
    prevent_initial_call=True,
)
def start_analysis(n_clicks, table_children, pathname):
    """Start optiSLang and run the simulation. Wait for the process to complete."""

    project = DashClient[{{ cookiecutter.__solution_definition_name }}].get_project(pathname)
    problem_setup_step = project.steps.problem_setup_step

    if n_clicks:
        # Update project properties file prior to the solve
        problem_setup_step.ui_placeholders = update_placeholders(table_children, problem_setup_step.placeholders)
        problem_setup_step.write_updated_properties_file()
        # Start analysis
        problem_setup_step.start_analysis()
        # Lock start analysis
        problem_setup_step.analysis_running = True
        # Wait until the analysis effectively starts
        while problem_setup_step.optislang_solve_status == "initial":
            time.sleep(1)
        return update_alerts(problem_setup_step), True, True
    else:
        raise PreventUpdate


@callback(
    Output("alert_messages", "children"),
    Output("start_analysis", "disabled"),
    Input("solve_interval_component", "n_intervals"),
    Input("start_analysis", "n_clicks"),
    State("url", "pathname"),
)
def update_alert_messages(n_intervals, n_clicks, pathname):
    """Display status badges."""

    project = DashClient[{{ cookiecutter.__solution_definition_name }}].get_project(pathname)
    problem_setup_step = project.steps.problem_setup_step

    if problem_setup_step.ansys_ecosystem_ready:
        status = problem_setup_step.get_long_running_method_state("start_analysis").status
        if status != MethodStatus.Running:
            problem_setup_step.analysis_running = False
        return update_alerts(problem_setup_step), problem_setup_step.analysis_running
    else:
        raise PreventUpdate



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

MONITORING_TABS = [
    {
        "label": "Project Summary",
        "tab_id": "project_summary_tab",
        "is_root": True,
        "is_system": False,
        "is_actor": False,
    },
    {"label": "Summary", "tab_id": "summary_tab", "is_root": False, "is_system": True, "is_actor": True},
    {"label": "Result Files", "tab_id": "result_files_tab", "is_root": True, "is_system": False, "is_actor": False},
    {"label": "Scenery", "tab_id": "scenery_tab", "is_root": True, "is_system": False, "is_actor": False},
    {"label": "Design Table", "tab_id": "design_table_tab", "is_root": True, "is_system": True, "is_actor": False},
    {"label": "Visualization", "tab_id": "visualization_tab", "is_root": True, "is_system": True, "is_actor": False},
    {"label": "Status Overview", "tab_id": "status_overview_tab", "is_root": True, "is_system": True, "is_actor": True},
]
